export interface FacePoint {
	x: number
	y: number
}

type PointType = "leftEye" | "rightEye" | "leftEyebrow" | "rightEyebrow" | "nose" | "mouth" | "contour"

export function indexFromTo(from: number, to: number) {
	const indexes: number[] = [];
	for (let i = from; i <= to; i++) {
		indexes.push(i);
	}
	return indexes;
}

interface PointGroup {
	/** 用于判断方向的两个点的索引(建议选取眼球中间的点) */
	directionPointIndex: [number, number];
	/** 左眼点的索引 */
	leftEyePointIndex: number[];
	/** 右眼点的索引 */
	rightEyePointIndex: number[];
	/** 左眉点的索引 */
	leftEyebrowPointIndex: number[];
	/** 右眉点的索引 */
	rightEyebrowPointIndex: number[];
	/** 嘴巴点的索引 */
	mouthPointIndex: number[];
	/** 鼻子的索引 */
	nosePointIndex: number[];
	/** 轮廓点的索引 */
	contourPointIndex: number[];
}

export abstract class FaceAlignmentResult {
	#points: FacePoint[]
	#group: PointGroup

	public constructor(points: FacePoint[], group: PointGroup) {
		this.#points = points;
		this.#group = group;
	}

	/** 关键点 */
	public get points() { return this.#points; }

	/** 获取特定的关键点 */
	public getPointsOf(type: PointType | PointType[]) {
		if (typeof type == "string") type = [type];
		const result: FacePoint[] = [];
		for (const t of type) {
			for (const idx of this.#group[`${t}PointIndex` as const]) {
				result.push(this.points[idx]);
			}
		}
		return result;
	}

	/** 方向 */
	public get direction() {
		const [{ x: x1, y: y1 }, { x: x2, y: y2 }] = this.#group.directionPointIndex.map(idx => this.points[idx]);
		return Math.atan2(y1 - y2, x2 - x1)
	}
}
